/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2022, 2023                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "BalanceAdjustments.h"

#include <pecunia/Money.h>
using pecunia::currency::Money;

#include <QObject>
using ::QObject;

#include "AccountCode.h"
using drn::accounting::AccountCode;
#include "AccountingErrors.h"
using drn::accounting::AccountError;
#include "Transaction.h"
using drn::accounting::Transaction;
#include <foundation/Optional.hpp>
using drn::foundation::presentationText;
#include <foundation/PresentationText.h>
using drn::foundation::presentationText;


Money drn::accounting::adjustAssetBalance(
	const AccountCode& asset,
	const Money& balance,
	const Transaction& txn
)
{
	if (txn.debiting() == asset)
		return balance + txn.amount();
	else if (txn.crediting() == asset)
		return balance - txn.amount();
	else
		throw AccountError{
			asset,
			QObject::tr(
				"Neither the account being credited (%1) nor debited (%2) in transaction %3 is "
					"the requested asset account, %4."
			)
				.arg(presentationText(txn.crediting()))
				.arg(presentationText(txn.debiting()))
				.arg(presentationText(txn.number()))
				.arg(presentationText(asset))
		};
}
