/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2022, 2023                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "PresentationText.h"
using drn::foundation::LocaleAware;
using drn::foundation::WithCurrency;

#include <ostream>
using std::ostream;

#include <pecunia/Codes.h>
using pecunia::currency::Iso4217Codes;
using pecunia::currency::minorUnitDigits;
using pecunia::currency::toStdString;
#include <pecunia/Money.h>
using pecunia::currency::Money;
#include <pecunia/MoneySize.hpp>
using pecunia::precision;
#include <QDate>
using ::QDate;
#include <QLocale>
using ::QLocale;
#include <QString>
using ::QString;


QString drn::foundation::presentationText(const Iso4217Codes& code)
{
	return QString::fromStdString(toStdString(code));
}

QString drn::foundation::presentationText(
	const Money& money,
	const LocaleAware isLocaleAware,
	const WithCurrency shouldAddCurrency
)
{
	const auto major{QString::number(money.major())};
	const auto minor{QString::number(money.minor())};
	const QLocale locale{};
	return QString{"%1%2%3%4"}
		.arg(major)
		.arg(isLocaleAware ? locale.decimalPoint() : QChar{'.'})
		.arg(minor, minorUnitDigits(money.code()) + precision, '0')
		.arg(shouldAddCurrency ? presentationText(money.code()) : QString{});
}

QString drn::foundation::presentationText(const QDate& value)
{
	return value.toString();
}

QString drn::foundation::presentationText(const qint8& value)
{
	return QString::number(value);
}

QString drn::foundation::presentationText(const quint8& value)
{
	return QString::number(value);
}

ostream& operator<<(ostream& o, const QString& value)
{
	return o << value.toStdString();
}

ostream& operator<<(ostream& o, const QDate& value)
{
	return o << value.toString();
}
