/*******************************************************************************
***  This file is part of Dux Rei Nummariae.                                 ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2022, 2023                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Affero General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Affero General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Affero General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#include "BankAccountSurvey.h"
using pecunia::currency::Money;
using std::map;
using std::set;
using drn::accounting::Transaction;
using drn::banking::ReconciledBankAccount;
using drn::budgeting::BudgetItemIdentifier;
using drn::surveying::BankAccountSurvey;

#include <QObject>
using ::QObject;

#include <utility>
using std::move;

#include <banking/BankingErrors.h>
using drn::banking::BankError;
#include <foundation/PresentationText.h>
using drn::foundation::presentationText;


BankAccountSurvey::BankAccountSurvey(
	ReconciledBankAccount reconciled,
	Money projectedBalance,
	map<BudgetItemIdentifier, set<Transaction>> usedBudget,
	map<BudgetItemIdentifier, Money> unusedBudget
) :
	reconciled_{move(reconciled)},
	projectedBalance_{move(projectedBalance)},
	usedBudget_{move(usedBudget)},
	unusedBudget_{move(unusedBudget)}
{
	if (this->reconciled_.balance().code() != this->projectedBalance_.code())
		throw BankError{
			this->reconciled_.bankAccount().bank_,
			QObject::tr(
				"A bank account survey must have the bank account currency '%1' the same as the "
					"projected balance currency '%2'."
			)
				.arg(presentationText(this->reconciled_.balance().code()))
				.arg(presentationText(this->projectedBalance_.code()))
		};
}

const ReconciledBankAccount& BankAccountSurvey::reconciled() const noexcept
{
	return this->reconciled_;
}

const Money& BankAccountSurvey::projectedBalance() const noexcept
{
	return this->projectedBalance_;
}

const map<BudgetItemIdentifier, set<Transaction>>& BankAccountSurvey::usedBudget() const noexcept
{
	return this->usedBudget_;
}

const map<BudgetItemIdentifier, Money>& BankAccountSurvey::unusedBudget() const noexcept
{
	return this->unusedBudget_;
}
